# SPDX-FileCopyrightText: 2012-2024 Efim Petelin.
#
# SPDX-License-Identifier: GPL-3.0-or-later

bl_info = {
    "name": "Render Overscan",
    "author": "Efim Petelin",
    "version": (0, 1, 14),
    "blender": (4, 2, 3),
    "location": "Properties > Output Properties",
    "category": "Render",
    "maintainer": "Efim Petelin efim.petelin@gmail.com",
    "description": "Applies and manages overscan in renders. Created with ChatGPT (v4.0)",
}


import bpy, blf, bgl
from .render_overscan import apply_overscan, restore_resolution, draw_callback_px, enable_draw_callback, disable_draw_callback

class OverscanPanel(bpy.types.Panel):
    """Creates a Panel in the Render properties for overscan controls."""
    bl_label = "Overscan Settings"
    bl_idname = "RENDER_PT_overscan"
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_category = 'output'
    bl_parent_id = 'RENDER_PT_format'
    
    def draw(self, context):
        layout = self.layout
        scene = context.scene
        camera = scene.camera

        layout.prop(scene, "overscan_size", text="Overscan Size")
        layout.prop_search(scene, "overscan_camera", scene, "objects", text="Camera", icon="CAMERA_DATA")

        # Button to select the active camera from the scene
        layout.operator("render.select_scene_camera", text="Select Scene Camera")

        # Toggle button for applying and restoring overscan
        if scene.get("overscan_enabled"):
            layout.operator("overscan.switch", text="Disable Overscan", icon="CANCEL")
        else:
            layout.operator("overscan.switch", text="Enable Overscan", icon="CHECKMARK")

class SelectSceneCameraOperator(bpy.types.Operator):
    """Selects the active camera from the scene."""
    bl_idname = "render.select_scene_camera"
    bl_label = "Select Scene Camera"
    
    def execute(self, context):
        scene = context.scene
        if scene.camera:
            scene.overscan_camera = scene.camera.name
            self.report({'INFO'}, f"Selected scene camera: {scene.camera.name}")
        else:
            self.report({'WARNING'}, "No active camera in scene!")
        return {'FINISHED'}

class OverscanSwitchOperator(bpy.types.Operator):
    """Toggles overscan on or off."""
    bl_idname = "overscan.switch"
    bl_label = "Toggle Overscan"
    
    def execute(self, context):
        scene = context.scene
        camera = bpy.data.objects.get(scene.overscan_camera) if scene.overscan_camera else scene.camera
        
        if scene.get("overscan_enabled"):
            restore_resolution(scene)
            scene["overscan_enabled"] = False
            disable_draw_callback()
            self.report({'INFO'}, "Overscan disabled")
        else:
            apply_overscan(scene, camera)
            scene["overscan_enabled"] = True
            enable_draw_callback(context)
            self.report({'INFO'}, "Overscan enabled")
        
        return {'FINISHED'}

def register():
    bpy.utils.register_class(OverscanPanel)
    bpy.utils.register_class(SelectSceneCameraOperator)
    bpy.utils.register_class(OverscanSwitchOperator)

    bpy.types.Scene.overscan_size = bpy.props.IntProperty(
        name="Overscan Size",
        description="Amount of overscan to apply in pixels",
        default=28,
        min=0,
        max=2000,
    )
    
    bpy.types.Scene.overscan_camera = bpy.props.StringProperty(
        name="Overscan Camera",
        description="Camera to apply overscan to",
        default="",
    )

def unregister():
    bpy.utils.unregister_class(OverscanPanel)
    bpy.utils.unregister_class(SelectSceneCameraOperator)
    bpy.utils.unregister_class(OverscanSwitchOperator)
    
    del bpy.types.Scene.overscan_size
    del bpy.types.Scene.overscan_camera

if __name__ == "__main__":
    register()
